@@ -20,7 +20,6 @@
 
 #include "../config.h"
 #include "audio.h"
-#include "utils.h"
 
 #define PA_BUFFER_LEN 4800
 #define CHANNELS 1
@@ -47,30 +46,29 @@ audio_write_callback (pa_stream * stream, size_t size, void *data)
   gshort *v;
   gint i;
 
-  //XXX: Until a better solution is found this corks the stream without using neither drain nor empty buffers (CPU)
   if (audio->release_frames > PA_BUFFER_LEN)
     {
-      audio_stop (audio, FALSE);
+      pa_stream_cork (audio->stream, 1, NULL, NULL);
       return;
     }
 
-  req_frames = size / 2;
+  req_frames = size >> 1;
   debug_print (2, "Writing %2d frames...\n", req_frames);
   pa_stream_begin_write (stream, &buffer, &size);
 
-  g_mutex_lock (&audio->mutex);
+  g_mutex_lock (&audio->control.mutex);
 
   if (!audio->sample->len)
     {
-      g_mutex_unlock (&audio->mutex);
+      g_mutex_unlock (&audio->control.mutex);
       pa_stream_cancel_write (stream);
       debug_print (2, "Canceled\n");
       return;
     }
 
-  if (audio->pos == audio->sample->len && !audio->loop)
+  if (audio->pos == audio->sample->len >> 1 && !audio->loop)
     {
-      g_mutex_unlock (&audio->mutex);
+      g_mutex_unlock (&audio->control.mutex);
       memset (buffer, 0, size);
       pa_stream_write (stream, buffer, size, NULL, 0, PA_SEEK_RELATIVE);
       audio->release_frames += req_frames;
@@ -80,7 +78,7 @@ audio_write_callback (pa_stream * stream, size_t size, void *data)
   v = buffer;
   for (i = 0; i < req_frames; i++)
     {
-      if (audio->pos < audio->sample->len)
+      if (audio->pos < audio->sample->len >> 1)
 	{
 	  *v = ((short *) audio->sample->data)[audio->pos];
 	  audio->pos++;
@@ -100,7 +98,7 @@ audio_write_callback (pa_stream * stream, size_t size, void *data)
       v++;
     }
 
-  g_mutex_unlock (&audio->mutex);
+  g_mutex_unlock (&audio->control.mutex);
 
   pa_stream_write (stream, buffer, i * 2, NULL, 0, PA_SEEK_RELATIVE);
 }
@@ -117,6 +115,7 @@ audio_stop (struct audio *audio, gboolean flush)
 
   debug_print (1, "Stopping audio...\n");
 
+  pa_threaded_mainloop_lock (audio->mainloop);
   if (flush)
     {
       operation = pa_stream_flush (audio->stream, NULL, NULL);
@@ -131,6 +130,7 @@ audio_stop (struct audio *audio, gboolean flush)
     {
       pa_operation_unref (operation);
     }
+  pa_threaded_mainloop_unlock (audio->mainloop);
 }
 
 void
@@ -147,16 +147,18 @@ audio_play (struct audio *audio)
 
   debug_print (1, "Playing audio...\n");
 
-  g_mutex_lock (&audio->mutex);
+  g_mutex_lock (&audio->control.mutex);
   audio->pos = 0;
   audio->release_frames = 0;
-  g_mutex_unlock (&audio->mutex);
+  g_mutex_unlock (&audio->control.mutex);
 
+  pa_threaded_mainloop_lock (audio->mainloop);
   operation = pa_stream_cork (audio->stream, 0, NULL, NULL);
   if (operation != NULL)
     {
       pa_operation_unref (operation);
     }
+  pa_threaded_mainloop_unlock (audio->mainloop);
 }
 
 static void
@@ -168,7 +170,7 @@ audio_set_sink_volume (pa_context * context, const pa_sink_input_info * info,
   if (info && pa_cvolume_valid (&info->volume))
     {
       gdouble v = pa_sw_volume_to_linear (pa_cvolume_avg (&info->volume));
-      audio->set_volume_gui_callback (v);
+      audio->volume_change_callback (v);
     }
 }
 
@@ -247,26 +249,31 @@ audio_context_callback (pa_context * context, void *data)
 }
 
 gint
-audio_init (struct audio *audio, void (*set_volume_gui_callback) (gdouble))
+audio_init (struct audio *audio, void (*volume_change_callback) (gdouble),
+	    void (*load_progress_callback) (gdouble))
 {
   pa_mainloop_api *api;
   gint err = 0;
 
   debug_print (1, "Initializing audio...\n");
 
-  audio->sample = g_array_new (FALSE, FALSE, sizeof (gshort));
+  audio->sample = g_byte_array_new ();
+  audio->frames = 0;
   audio->loop = FALSE;
-  audio->mainloop = pa_glib_mainloop_new (NULL);
-  api = pa_glib_mainloop_get_api (audio->mainloop);
+  audio->mainloop = pa_threaded_mainloop_new ();
+  api = pa_threaded_mainloop_get_api (audio->mainloop);
   audio->context = pa_context_new (api, PACKAGE);
   audio->stream = NULL;
   audio->index = PA_INVALID_INDEX;
-  audio->set_volume_gui_callback = set_volume_gui_callback;
+  audio->volume_change_callback = volume_change_callback;
+  audio->control.callback = load_progress_callback;
+  audio->name = malloc (PATH_MAX);
+  audio->name[0] = 0;
 
   if (pa_context_connect (audio->context, NULL, PA_CONTEXT_NOFLAGS, NULL) < 0)
     {
       pa_context_unref (audio->context);
-      pa_glib_mainloop_free (audio->mainloop);
+      pa_threaded_mainloop_free (audio->mainloop);
       audio->mainloop = NULL;
       err = -1;
     }
@@ -274,6 +281,9 @@ audio_init (struct audio *audio, void (*set_volume_gui_callback) (gdouble))
     {
       pa_context_set_state_callback (audio->context, audio_context_callback,
 				     audio);
+      pa_threaded_mainloop_start (audio->mainloop);
+
+
     }
 
   return err;
@@ -285,7 +295,8 @@ audio_destroy (struct audio *audio)
   debug_print (1, "Destroying audio...\n");
 
   audio_stop (audio, TRUE);
-  g_array_free (audio->sample, TRUE);
+
+  g_byte_array_free (audio->sample, TRUE);
   if (audio->stream)
     {
       pa_stream_unref (audio->stream);
@@ -295,9 +306,12 @@ audio_destroy (struct audio *audio)
   if (audio->mainloop)
     {
       pa_context_unref (audio->context);
-      pa_glib_mainloop_free (audio->mainloop);
+      pa_threaded_mainloop_stop (audio->mainloop);
+      pa_threaded_mainloop_free (audio->mainloop);
       audio->mainloop = NULL;
     }
+
+  g_free (audio->name);
 }
 
 gboolean
@@ -309,10 +323,11 @@ audio_check (struct audio *audio)
 void
 audio_reset_sample (struct audio *audio)
 {
-  g_mutex_lock (&audio->mutex);
-  g_array_set_size (audio->sample, 0);
+  g_mutex_lock (&audio->control.mutex);
+  g_byte_array_set_size (audio->sample, 0);
+  audio->frames = 0;
   audio->pos = 0;
-  g_mutex_unlock (&audio->mutex);
+  g_mutex_unlock (&audio->control.mutex);
 }
 
 void
