@@ -20,23 +20,23 @@
 
 #include <glib.h>
 #include <pulse/pulseaudio.h>
-#include <pulse/glib-mainloop.h>
+#include "utils.h"
 
 struct audio
 {
-  GArray *sample;
-  gint frames;
+  GByteArray *sample;
+  guint frames;
   gboolean loop;
-  pa_glib_mainloop *mainloop;
+  pa_threaded_mainloop *mainloop;
   pa_context *context;
   pa_stream *stream;
   gint pos;
   pa_cvolume volume;
   uint32_t index;
-  void (*set_volume_gui_callback) (gdouble);
+  void (*volume_change_callback) (gdouble);
   gint release_frames;
-  GMutex mutex;
-  gboolean load_active;
+  struct job_control control;
+  gchar *name;
 };
 
 void audio_play (struct audio *);
@@ -45,7 +45,7 @@ void audio_stop (struct audio *, gboolean);
 
 gboolean audio_check (struct audio *);
 
-gint audio_init (struct audio *, void (*)(gdouble));
+gint audio_init (struct audio *, void (*)(gdouble), void (*)(gdouble));
 
 void audio_destroy (struct audio *);
 
