@@ -21,10 +21,22 @@
 #include <glib.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include "utils.h"
 
-#define LOAD_BUFFER_LEN 4800
+#ifndef SAMPLE_H
+#define SAMPLE_H
 
-size_t sample_save (GArray *, gchar *);
+#define LOAD_BUFFER_LEN 4800	// In guint16 frames; 9600 B; 0.1 ms
 
-size_t sample_load (GArray *, GMutex *, gboolean *, gchar *, gint *,
-		    void (*)(gdouble));
+gint sample_wave (GByteArray *, GByteArray *, struct job_control *);
+
+gint sample_raw (GByteArray *, GByteArray *, struct job_control *);
+
+gint sample_save (const gchar *, GByteArray *, struct job_control *);
+
+gint sample_load (const gchar *, GByteArray *, struct job_control *);
+
+gint sample_load_with_frames (const gchar *, GByteArray *,
+			      struct job_control *, guint *);
+
+#endif
