@@ -18,16 +18,15 @@
  *   along with Elektroid. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <glib.h>
 #include <stdio.h>
 #include <wordexp.h>
+#include <errno.h>
 #include "utils.h"
 
 #define DEBUG_SHORT_HEX_LEN 64
 #define DEBUG_FULL_HEX_THRES 3
 
-#define REG_TYPE "emblem-music-symbolic"
-#define DIR_TYPE "folder-visiting-symbolic"
+#define KIB 1024
 
 gint debug_level;
 
@@ -149,7 +148,7 @@ get_ext (const gchar * name)
   int i = namelen;
   const gchar *ext = &name[namelen];
 
-  while (*(ext - 1) != '.' && i > 0)
+  while (i > 0 && *(ext - 1) != '.')
     {
       ext--;
       i--;
@@ -165,36 +164,20 @@ get_ext (const gchar * name)
     }
 }
 
-char
-get_type_from_inventory_icon (const gchar * icon)
+gchar *
+get_expanded_dir (const char *exp)
 {
-  gchar type;
+  wordexp_t exp_result;
+  size_t n;
+  gchar *exp_dir = malloc (PATH_MAX);
 
-  if (strcmp (icon, REG_TYPE) == 0)
-    {
-      type = ELEKTROID_FILE;
-    }
-  else
-    {
-      type = ELEKTROID_DIR;
-    }
-  return type;
-}
+  wordexp (exp, &exp_result, 0);
+  n = PATH_MAX - 1;
+  strncpy (exp_dir, exp_result.we_wordv[0], n);
+  exp_dir[PATH_MAX - 1] = 0;
+  wordfree (&exp_result);
 
-const gchar *
-get_inventory_icon_from_type (char type)
-{
-  const gchar *icon;
-
-  if (type == ELEKTROID_FILE)
-    {
-      icon = REG_TYPE;
-    }
-  else
-    {
-      icon = DIR_TYPE;
-    }
-  return icon;
+  return exp_dir;
 }
 
 char *
@@ -202,7 +185,6 @@ get_local_startup_path (const gchar * local_dir)
 {
   DIR *dir;
   gchar *startup_path = NULL;
-  wordexp_t exp_result;
 
   if (local_dir)
     {
@@ -214,20 +196,187 @@ get_local_startup_path (const gchar * local_dir)
 	}
       else
 	{
-	  error_print ("Unable to open dir %s\n", local_dir);
+	  error_print ("Unable to open dir '%s'\n", local_dir);
 	}
       closedir (dir);
     }
 
   if (!startup_path)
     {
-      wordexp ("~", &exp_result, 0);
-      startup_path = malloc (PATH_MAX);
-      strcpy (startup_path, exp_result.we_wordv[0]);
-      wordfree (&exp_result);
+      startup_path = get_expanded_dir ("~");
     }
 
-  debug_print (1, "Using %s as local dir...\n", startup_path);
+  debug_print (1, "Using '%s' as local dir...\n", startup_path);
 
   return startup_path;
 }
+
+void
+free_msg (gpointer msg)
+{
+  g_byte_array_free ((GByteArray *) msg, TRUE);
+}
+
+gchar *
+get_item_name (struct item *item)
+{
+  return strdup (item->name);
+}
+
+gchar *
+get_item_index (struct item *item)
+{
+  gchar *index;
+
+  if (item->type == ELEKTROID_DIR)
+    {
+      index = get_item_name (item);
+    }
+  else
+    {
+      index = malloc (LABEL_MAX);
+      snprintf (index, LABEL_MAX, "%d", item->index);
+    }
+
+  return index;
+}
+
+guint
+next_item_iterator (struct item_iterator *iter)
+{
+  return iter->next (iter);
+}
+
+void
+free_item_iterator (struct item_iterator *iter)
+{
+  if (iter->free)
+    {
+      iter->free (iter->data);
+    }
+}
+
+gint
+copy_item_iterator (struct item_iterator *dst, struct item_iterator *src,
+		    gboolean cached)
+{
+  return src->copy (dst, src, cached);
+}
+
+gint
+load_file (const char *path, GByteArray * array, struct job_control *control)
+{
+  FILE *file;
+  long size;
+  gint res;
+
+  file = fopen (path, "rb");
+
+  if (!file)
+    {
+      return -errno;
+    }
+
+  res = 0;
+
+  if (fseek (file, 0, SEEK_END))
+    {
+      error_print ("Unexpected value\n");
+      res = -errno;
+      goto end;
+    }
+
+  size = ftell (file);
+  rewind (file);
+
+  g_byte_array_set_size (array, size);
+
+  if (fread (array->data, 1, size, file) == size)
+    {
+      debug_print (1, "%zu bytes read\n", size);
+    }
+  else
+    {
+      error_print ("Error while reading from file %s\n", path);
+      res = -errno;
+    }
+
+end:
+  fclose (file);
+  return res;
+}
+
+gint
+save_file_char (const gchar * path, const guint8 * data, ssize_t len)
+{
+  gint res;
+  long bytes;
+  FILE *file;
+
+  file = fopen (path, "w");
+
+  if (!file)
+    {
+      return -errno;
+    }
+
+  debug_print (1, "Saving file %s...\n", path);
+
+  res = 0;
+  bytes = fwrite (data, 1, len, file);
+  if (bytes == len)
+    {
+      debug_print (1, "%zu bytes written\n", bytes);
+    }
+  else
+    {
+      error_print ("Error while writing to file %s\n", path);
+      res = -EIO;
+    }
+
+  fclose (file);
+
+  return res;
+}
+
+gint
+save_file (const gchar * path, GByteArray * array,
+	   struct job_control *control)
+{
+  return save_file_char (path, array->data, array->len);
+}
+
+gchar *
+get_human_size (guint size, gboolean with_space)
+{
+  gchar *label = malloc (LABEL_MAX);
+  gchar *space = with_space ? " " : "";
+
+  if (size < KIB)
+    {
+      snprintf (label, LABEL_MAX, "%d%sB", size, space);
+    }
+  else if (size < KIB * KIB)
+    {
+      snprintf (label, LABEL_MAX, "%.2f%sKiB", size / (double) KIB, space);
+    }
+  else if (size < KIB * KIB * KIB)
+    {
+      snprintf (label, LABEL_MAX, "%.2f%sMiB", size / (double) (KIB * KIB),
+		space);
+    }
+  else
+    {
+      snprintf (label, LABEL_MAX, "%.2f%sGiB",
+		size / (double) (KIB * KIB * KIB), space);
+    }
+
+  return label;
+}
+
+void
+set_job_control_progress (struct job_control *control, gdouble p)
+{
+  gdouble v = (double) control->part / control->parts + p / control->parts;
+  control->callback (v);
+}
